package openfuture.masterdata;

import java.util.*;

/**
 * This is a default implementation for a Master Data provider.
 *
 * Creation date: (09.09.00 21:59:30)
 * @author: Markus Giebeler
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.<p>
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.<p>
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA<br>
 * http://www.gnu.org/copyleft/lesser.html"
 */
public class DefaultMasterDataProvider implements IMasterDataProvider, ILanguageDependent {
  /** Field service (IMasterDataService) is used to connect provider to service */
  private IMasterDataService service;

	/** Field masterDataCache (Hashtable) is used to cache the master data in the provider */
  private Hashtable masterDataCache;

	/** Field language (String) is used to identify the language of the provider */
  private String language;
/**
 * Constructor
 */
public DefaultMasterDataProvider() {
	super();
}
/**
 * fetch master data form the service
 * 
 * Creation date: (11.09.00 09:53:46)
 * @return openfuture.masterdata.IMasterDataContainer
 * @param type java.lang.String
 */
private IMasterDataContainer fetchMasterData(String type) {
	String key = getKey(type, getLanguage());
	IMasterDataContainer container = getService().getMasterData(type, getLanguage());

	// if container has no data, get master data for default language.
	if(container.getData().size()==0) {
		container = getService().getMasterData(type, Locale.getDefault().getLanguage());
	}

	// store container in the master data cache
	getMasterDataCache().put(key, container);
	return container;
}
/**
 * get a Hashtable key for a given type and language
 * 
 * Creation date: (11.09.00 14:48:24)
 * @return java.lang.String
 * @param type java.lang.String
 * @param language java.lang.String
 */
private String getKey(String type, String language) {
	// use the following version to keep master data in the cache
	// return language+"-"+type;

	// use this version to keep master data only in one language
	return type;
}
/**
  * Return contents of field language
  * 
  * @return (String) contents of field language
  */
public String getLanguage() {
	if(language==null) {
		language = Locale.getDefault().getLanguage();
	}
	return language;
}
/**
 * get some master data from the cache; if no data available in the provider,
 * data are fetched from the service
 *
 * @param type (String) the md type
 */
public Vector getMasterData(String type) {
	String key = getKey(type, getLanguage());
	IMasterDataContainer container = (IMasterDataContainer) getMasterDataCache().get(key);

	// if no master data for the language in the cache, fetch some data from the service
	if(container == null || !container.hasMasterData(getLanguage())) {
		container = fetchMasterData(type);
	}
	
	return container.getData();
}
/**
 * get some master data from the cache for a given language
 *
 * @param type (String) the md type
 */
public IMasterData getMasterData(String type, String id) {
	Enumeration allData = getMasterData(type).elements();

	while(allData.hasMoreElements()) {
		IMasterData data = (IMasterData) allData.nextElement();
		if(data.getId().equalsIgnoreCase(id)) {
			return data;
		}
	}
	
	return null;
}
/**
 * Return contents of field masterDataCache
 * 
 * @return (Hashtable) contents of field masterDataCache
 */
public Hashtable getMasterDataCache() {
	if (masterDataCache == null) {
		setMasterDataCache(new Hashtable());
	}
	return masterDataCache;
}
/**
 * Return contents of field service
 * 
 * @return (IMasterDataService) contents of field service
 */
public final IMasterDataService getService() {
	return service;
}
/**
 * Set contents of field language to the given value
 * 
 * @param newValue (String) new value for field language
 */
public void setLanguage(String newValue) {
	this.language = newValue;
}
/**
 * Set contents of field masterDataCache to the given value
 * 
 * @param newValue (Hashtable) new value for field masterDataCache
 */
public void setMasterDataCache(Hashtable newValue) {
	masterDataCache = newValue;
}
/**
 * Set contents of field service to the given value
 * 
 * @param newValue (IMasterDataService) new value for field service
 */
public final void setService(IMasterDataService newValue) {
	this.service = newValue;
}
}
